@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20764 $
- * $Author: rjongbloed $
- * $Date: 2008-08-29 02:15:19 +0000 (Fri, 29 Aug 2008) $
+ * $Revision: 21525 $
+ * $Author: csoutheren $
+ * $Date: 2008-11-14 11:01:53 +0000 (Fri, 14 Nov 2008) $
  */
 
 #ifdef __GNUC__
@@ -179,6 +179,38 @@ PObject::Comparison PURL::Compare(const PObject & obj) const
   return urlString.Compare(((const PURL &)obj).urlString);
 }
 
+PURL::PURL(const PURL & other)
+{
+  CopyContents(other);
+}
+
+PURL & PURL::operator=(const PURL & other)
+{
+  CopyContents(other);
+  return *this;
+}
+
+void PURL::CopyContents(const PURL & other)
+{
+  urlString    = other.urlString;
+  scheme       = other.scheme;
+  username     = other.username;
+  password     = other.password;
+  hostname     = other.hostname;
+  port         = other.port;
+  portSupplied = other.portSupplied;
+  relativePath = other.relativePath;
+  pathStr      = other.pathStr;
+  path         = other.path;
+  fragment     = other.fragment;
+
+  paramVars    = other.paramVars;
+  paramVars.MakeUnique();
+
+  queryVars    = other.queryVars;
+  queryVars.MakeUnique();
+
+}
 
 PINDEX PURL::HashFunction() const
 {
@@ -634,7 +666,7 @@ PString PURL::LegacyAsString(PURL::UrlFormat fmt, const PURLLegacyScheme * schem
     }
 
     if (schemeInfo->hasHostPort) {
-      if (hostname.Find(':') != P_MAX_INDEX)
+      if (hostname.Find(':') != P_MAX_INDEX && hostname[0] != '[')
         str << '[' << hostname << ']';
       else
         str << hostname;
