@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20428 $
- * $Author: rjongbloed $
- * $Date: 2008-06-10 13:59:48 +0200 (mar 10 jun 2008) $
+ * $Revision: 21397 $
+ * $Author: csoutheren $
+ * $Date: 2008-10-21 07:46:34 +0000 (Tue, 21 Oct 2008) $
  */
 
 #include <ptlib.h>
@@ -2559,30 +2559,35 @@ PRegularExpression::PRegularExpression()
 PRegularExpression::PRegularExpression(const PString & pattern, int flags)
 {
   expression = NULL;
-  Compile(pattern, flags);
+  bool b = Compile(pattern, flags);
+  PAssert(b, PString("regular expression compile failed : " + GetErrorText()));
 }
 
 
 PRegularExpression::PRegularExpression(const char * pattern, int flags)
 {
   expression = NULL;
-  Compile(pattern, flags);
+  bool b = Compile(pattern, flags);
+  PAssert(b, PString("regular expression compile failed : " + GetErrorText()));
 }
 
 PRegularExpression::PRegularExpression(const PRegularExpression & from)
 {
   expression   = NULL;
-  patternSaved = from.patternSaved;
-  flagsSaved   = from.flagsSaved;
-  Compile(patternSaved, flagsSaved);
+  bool b = Compile(from.patternSaved, from.flagsSaved);
+  PAssert(b, PString("regular expression compile failed : " + GetErrorText()));
 }
 
 PRegularExpression & PRegularExpression::operator =(const PRegularExpression & from)
 {
-  expression   = NULL;
-  patternSaved = from.patternSaved;
-  flagsSaved   = from.flagsSaved;
-  Compile(patternSaved, flagsSaved);
+  if (expression != from.expression) {
+    if (expression != NULL && expression != from.expression) {
+      regfree(regexpression);
+      delete regexpression;
+    }
+    expression = NULL;
+  }
+  Compile(from.patternSaved, from.flagsSaved);
   return *this;
 }
 
