@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21322 $
+ * $Revision: 21995 $
  * $Author: rjongbloed $
- * $Date: 2008-10-14 06:43:48 +0000 (Tue, 14 Oct 2008) $
+ * $Date: 2009-02-04 23:59:32 +0000 (Wed, 04 Feb 2009) $
  */
 
 #include <ptlib.h>
@@ -86,7 +86,10 @@ static PIPSocket::Address broadcast4(INADDR_BROADCAST);
 static PIPSocket::Address any4(INADDR_ANY);
 static in_addr inaddr_empty;
 #if P_HAS_IPV6
+static int defaultIPv6ScopeId = 0; 
+
 static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
+static PIPSocket::Address broadcast6(16,(const BYTE *)"\377\002\0\0\0\0\0\0\0\0\0\0\0\0\0\001"); // IPV6 multicast address
 static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"); 
 #endif
 
@@ -116,6 +119,15 @@ void PIPSocket::SetDefaultIpAddressFamilyV6()
   SetDefaultIpAddressFamily(PF_INET6);
 }
 
+void PIPSocket::SetDefaultV6ScopeId(int scopeId) 
+{
+  defaultIPv6ScopeId = scopeId;
+}; 
+
+int PIPSocket::GetDefaultV6ScopeId()
+{
+  return defaultIPv6ScopeId;
+}
 
 PBoolean PIPSocket::IsIpAddressFamilyV6Supported()
 {
@@ -172,7 +184,7 @@ Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
     addr6->sin6_addr = ip;
     addr6->sin6_port = htons(port);
     addr6->sin6_flowinfo = 0;
-    addr6->sin6_scope_id = 0; // Should be set to the right interface....
+	addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
   }
   else {
     sockaddr_in * addr4 = (sockaddr_in *)&storage;
@@ -492,10 +504,10 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
   PINDEX len = key.GetLength();
 
   // Check for a legal hostname as per RFC952
+  // but drop the requirement for leading alpha as per RFC 1123
   if (key.IsEmpty() ||
       key.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.") != P_MAX_INDEX ||
-      key[len-1] == '-' ||
-      !isalpha(key[0])) {
+      key[len-1] == '-') {
     PTRACE(3, "Socket\tIllegal RFC952 characters in DNS name \"" << key << '"');
     return NULL;
   }
@@ -596,12 +608,11 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     SetAt(key, host);
   }
 
-  if (host->GetHostAddress() == 0) {
-    PTRACE(4, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
-    return NULL;
-  }
+  if (host->GetHostAddress().IsValid())
+    return host;
 
-  return host;
+  PTRACE(4, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
+  return NULL;
 }
 
 
@@ -732,10 +743,7 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
     SetAt(key, host);
   }
 
-  if (host->GetHostAddress() == 0)
-    return NULL;
-
-  return host;
+  return host->GetHostAddress().IsValid() ? host : NULL;
 }
 
 
@@ -1127,7 +1135,7 @@ PString PIPSocket::GetHostName(const PString & hostname)
 {
   // lookup the host address using inet_addr, assuming it is a "." address
   Address temp = hostname;
-  if (temp != 0)
+  if (temp.IsValid())
     return GetHostName(temp);
 
   PString canonicalname;
@@ -1140,13 +1148,18 @@ PString PIPSocket::GetHostName(const PString & hostname)
 
 PString PIPSocket::GetHostName(const Address & addr)
 {
-  if (addr == 0)
+  if (!addr.IsValid())
     return addr.AsString();
 
   PString hostname;
   if (pHostByAddr().GetHostName(addr, hostname))
     return hostname;
 
+#if P_HAS_IPV6
+  if (addr.GetVersion() == 6)
+    return '[' + addr.AsString() + ']';
+#endif
+
   return addr.AsString();
 }
 
@@ -1186,7 +1199,7 @@ PStringArray PIPSocket::GetHostAliases(const PString & hostname)
 
   // lookup the host address using inet_addr, assuming it is a "." address
   Address addr = hostname;
-  if (addr != 0)
+  if (addr.IsValid())
     pHostByAddr().GetHostAliases(addr, aliases);
   else
     pHostByName().GetHostAliases(hostname, aliases);
@@ -1223,6 +1236,18 @@ PBoolean PIPSocket::GetLocalAddress(Address & addr)
 }
 
 
+PBoolean PIPSocket::GetLocalAddress(PIPSocketAddressAndPort & addr)
+{
+  Address ip;
+  WORD port;
+  if (!GetLocalAddress(ip, port))
+    return false;
+
+  addr.SetAddress(ip, port);
+  return true;
+}
+
+
 PBoolean PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
 {
 #if P_HAS_IPV6
@@ -1279,6 +1304,19 @@ PBoolean PIPSocket::GetPeerAddress(Address & addr)
   return GetPeerAddress(addr, portNum);
 }
 
+
+PBoolean PIPSocket::GetPeerAddress(PIPSocketAddressAndPort & addr)
+{
+  Address ip;
+  WORD port;
+  if (!GetPeerAddress(ip, port))
+    return false;
+
+  addr.SetAddress(ip, port);
+  return true;
+}
+
+
 PBoolean PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
 {
 #if P_HAS_IPV6
@@ -1462,12 +1500,6 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 
 #if P_HAS_IPV6
   Psockaddr bind_sa(bindAddr, port); 
-
-  if (IsOpen()) {
-    int socketType;
-    if (!GetOption(SO_TYPE, socketType, SOL_SOCKET) || bind_sa->sa_family != socketType)
-      Close();
-  }
 #endif
 
   if (!IsOpen()) {
@@ -1482,11 +1514,16 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
   }
   
 #ifndef P_BEOS
-  // attempt to listen
-  if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
-    os_close();
-    return PFalse;
+#if P_HAS_IPV6
+  if(bind_sa->sa_family != AF_INET6)
+  {
+	 // attempt to listen
+	 if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
+	   os_close();
+	   return PFalse;
+	 }
   }
+#endif
 #else
   // attempt to listen
   int value = reuse == CanReuseAddress ? 1 : 0;
@@ -1505,7 +1542,26 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
       return PFalse;
 
     port = sa.GetPort();
-    return PTrue;
+
+    if (!bindAddr.IsMulticast())
+      return true;
+
+    // Probably move this stuff into a separate method
+    if (bindAddr.GetVersion() == 4) {
+      struct ip_mreq  mreq;
+      mreq.imr_multiaddr = in_addr(bindAddr);
+      mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+      if (SetOption(IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq), IPPROTO_IP)) {
+        PTRACE(4, "Socket\tJoined multicast group");
+        return true;
+      }
+      PTRACE(1, "Socket\tFailed to join multicast group");
+    }
+    else {
+      PTRACE(1, "Socket\tIPV6 Multicast join not implemented yet");
+    }
+
+    return false;
   }
 
 #else
@@ -1531,7 +1587,18 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
     socklen_t size = sizeof(sin);
     if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
       port = ntohs(sin.sin_port);
-      return PTrue;
+
+	  if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
+        return true;
+
+      struct ip_mreq  mreq;
+      mreq.imr_multiaddr.s_addr = sin.sin_addr.s_addr;
+      mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+      if (SetOption(IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq))) {
+        PTRACE(4, "Socket\tJoined multicast group");
+        return true;
+      }
+      PTRACE(1, "Socket\tFailed to join multicast group");
     }
   }
 
@@ -1542,12 +1609,6 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetLoopback()
-{
-  return loopback4;
-}
-
-
 #if P_HAS_IPV6
 
 /// Check for v4 mapped i nv6 address ::ffff:a.b.c.d
@@ -1559,15 +1620,36 @@ PBoolean PIPSocket::Address::IsV4Mapped() const
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetLoopback6()
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int version)
 {
-  return loopback6;
+  return version == 6 ? loopback6 : loopback4;
 }
 
-
-const PIPSocket::Address & PIPSocket::Address::GetAny6()
+const PIPSocket::Address & PIPSocket::Address::GetAny(int version)
 {
-  return any6;
+  return version == 6 ? any6 : any4;
+}
+
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int version)
+{
+  return version == 6 ? broadcast6 : broadcast4;
+}
+
+#else
+
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int)
+{
+  return loopback4;
+}
+
+const PIPSocket::Address & PIPSocket::Address::GetAny(int)
+{
+  return any4;
+}
+
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int)
+{
+  return broadcast4;
 }
 
 #endif
@@ -1579,15 +1661,14 @@ PBoolean PIPSocket::Address::IsAny() const
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetBroadcast()
-{
-  return broadcast4;
-}
-
-
 PIPSocket::Address::Address()
 {
-  *this = loopback4;
+#if P_HAS_IPV6
+	if(defaultIpAddressFamily == AF_INET6)
+		*this = loopback6;
+	else
+#endif
+		*this = loopback4;
 }
 
 
@@ -1760,10 +1841,13 @@ bool PIPSocket::Address::operator==(in_addr & addr) const
 
 bool PIPSocket::Address::operator==(DWORD dw) const
 {
-  if (dw != 0)
+  if (dw == 0)
+    return !IsValid();
+
+  if (version == 4)
     return (DWORD)*this == dw;
 
-  return !IsValid();
+  return *this == Address(dw);
 }
 
 
@@ -1774,7 +1858,7 @@ PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 }
 
 
-PString PIPSocket::Address::AsString() const
+PString PIPSocket::Address::AsString(bool bracketIPv6) const
 {
 #if defined(P_VXWORKS)
   char ipStorage[INET_ADDR_LEN];
@@ -1790,7 +1874,9 @@ PString PIPSocket::Address::AsString() const
     if (percent != P_MAX_INDEX)
       str[percent] = '\0';
     str.MakeMinimumSize();
-    return str;
+    if (bracketIPv6)
+      return '[' + str + ']';
+	return str;
   }
 #endif // P_HAS_IPV6
 # if defined(P_HAS_INET_NTOP)
@@ -1818,6 +1904,12 @@ PBoolean PIPSocket::Address::FromString(const PString & ipAndInterface)
   if (!dotNotation.IsEmpty()) {
 #if P_HAS_IPV6
 
+    // Find out if string is in brackets [], as in ipv6 address
+    PINDEX lbracket = dotNotation.Find('[');
+    PINDEX rbracket = dotNotation.Find(']', lbracket);
+    if (lbracket != P_MAX_INDEX && rbracket != P_MAX_INDEX)
+      dotNotation = dotNotation(lbracket+1, rbracket-1);
+
     struct addrinfo *res = NULL;
     struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
 
@@ -1983,12 +2075,24 @@ PBoolean PIPSocket::Address::IsBroadcast() const
 {
 #if P_HAS_IPV6
   if (version == 6) // In IPv6, no broadcast exist. Only multicast
-    return PFalse;
+    return *this == broadcast6; // multicast address as as substitute
 #endif
 
   return *this == broadcast4;
 }
 
+
+PBoolean PIPSocket::Address::IsMulticast() const
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return IN6_IS_ADDR_MULTICAST(&v.six);
+#endif
+
+  return IN_MULTICAST(ntohl(v.four.s_addr));
+}
+
+
 PBoolean PIPSocket::Address::IsRFC1918() const 
 { 
 #if P_HAS_IPV6
@@ -2396,7 +2500,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
   
 #if P_HAS_IPV6
   
-  Psockaddr sa(broadcast ? Address::GetBroadcast() : addr, port);
+  Psockaddr sa(broadcast ? Address::GetBroadcast(addr.GetVersion()) : addr, port);
   PBoolean ok = os_sendto(buf, len, 0, sa, sa.GetSize()) != 0;
   
 #else
@@ -2423,30 +2527,44 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
 //////////////////////////////////////////////////////////////////////////////
 // PUDPSocket
 
-PUDPSocket::PUDPSocket(WORD newPort)
+PUDPSocket::PUDPSocket(WORD newPort, int iAddressFamily)
 {
   sendPort = 0;
   SetPort(newPort);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
-PUDPSocket::PUDPSocket(PQoS * qos, WORD newPort)
+PUDPSocket::PUDPSocket(PQoS * qos, WORD newPort, int iAddressFamily)
+#if P_HAS_IPV6
+  : sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
+    lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
+#else
+  : sendAddress(loopback4),
+    lastReceiveAddress(loopback4)
+#endif
 {
   if (qos != NULL)
       qosSpec = *qos;
   sendPort = 0;
   SetPort(newPort);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
 
-PUDPSocket::PUDPSocket(const PString & service, PQoS * qos)
+PUDPSocket::PUDPSocket(const PString & service, PQoS * qos, int iAddressFamily)
+#if P_HAS_IPV6
+  : sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
+    lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
+#else
+  : sendAddress(loopback4),
+    lastReceiveAddress(loopback4)
+#endif
 {
   if (qos != NULL)
       qosSpec = *qos;
   sendPort = 0;
   SetPort(service);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
 
@@ -2667,18 +2785,16 @@ PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 
 #define COULD_HAVE_QOS
 
-static PBoolean CheckOSVersion()
+static PBoolean CheckOSVersionFor(DWORD major, DWORD minor)
 {
-    OSVERSIONINFO versInfo;
-    ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
-    versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    if (GetVersionEx(&versInfo))
-    {
-        if (versInfo.dwMajorVersion > 5 ||
-           (versInfo.dwMajorVersion == 5 &&
-            versInfo.dwMinorVersion > 0))
-          return PTrue;
-    }
+  OSVERSIONINFO versInfo;
+  ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
+  versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+  if (GetVersionEx(&versInfo)) {
+    if (versInfo.dwMajorVersion > major ||
+       (versInfo.dwMajorVersion == major && versInfo.dwMinorVersion >= minor))
+      return PTrue;
+  }
   return PFalse;
 }
 
@@ -2689,7 +2805,7 @@ static PBoolean CheckOSVersion()
 PBoolean PUDPSocket::OpenSocket()
 {
 #ifdef COULD_HAVE_QOS
-  if (CheckOSVersion()) 
+  if (CheckOSVersionFor(5,1)) 
     return OpenSocketGQOS(AF_INET, SOCK_DGRAM, 0);
 #endif
 
@@ -2699,7 +2815,7 @@ PBoolean PUDPSocket::OpenSocket()
 PBoolean PUDPSocket::OpenSocket(int ipAdressFamily)
 {
 #ifdef COULD_HAVE_QOS
-  if (CheckOSVersion()) 
+  if (CheckOSVersionFor(5,1)) 
     return OpenSocketGQOS(ipAdressFamily, SOCK_DGRAM, 0);
 #endif
 
@@ -2764,23 +2880,31 @@ PBoolean PICMPSocket::OpenSocket(int)
 
 //////////////////////////////////////////////////////////////////////////////
 
-PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD _port, char _sep)
+PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD port, char separator)
 {
-  sep  = _sep;
-  port = _port;
+  m_separator= separator;
+  m_port = port;
 
-  PINDEX pos = str.Find(sep);
+  PINDEX pos = str.Find(separator);
   if (pos != P_MAX_INDEX) {
-    port    = (WORD)str.Mid(pos+1).AsInteger();
-    if (!PIPSocket::GetHostAddress(str.Left(pos), address))
-      return PFalse;
+    m_port    = (WORD)str.Mid(pos+1).AsInteger();
+    if (!PIPSocket::GetHostAddress(str.Left(pos), m_address))
+      return false;
   }
-  else if (port == 0) 
-    return PFalse;
-  else if (!PIPSocket::GetHostAddress(str, address))
-    return PFalse;
+  else if (m_port == 0) 
+    return false;
+  else if (!PIPSocket::GetHostAddress(str, m_address))
+    return false;
 
-  return PTrue;
+  return true;
+}
+
+
+void PIPSocketAddressAndPort::SetAddress(const PIPSocket::Address & addr, WORD port)
+{
+  m_address = addr;
+  if (port != 0)
+    m_port = port;
 }
 
 
