@@ -23,9 +23,9 @@
  *
  * Contributor(s): Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
- * $Revision: 20498 $
- * $Author: shorne $
- * $Date: 2008-06-26 01:05:26 +0000 (Thu, 26 Jun 2008) $
+ * $Revision: 21747 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-10 02:06:07 +0000 (Wed, 10 Dec 2008) $
  */
 
 #ifdef __GNUC__
@@ -79,20 +79,19 @@ ostream & operator<<(ostream & strm, PVideoDevice::VideoFormat fmt)
 
 
 //Colour format bit per pixel table.
-// These are in rough order of colour gamut size
+// These are in rough order of colour gamut size and "popularity"
 static struct {
   const char * colourFormat;
   unsigned     bitsPerPixel;
 } colourFormatBPPTab[] = {
-  { "YUV420",  12 },
   { "YUV420P", 12 },
   { "I420",    12 },
+  { "IYUV",    12 },
+  { "YUV420",  12 },
   { "RGB32",   32 },
   { "BGR32",   32 },
   { "RGB24",   24 },
   { "BGR24",   24 },
-  { "MJPEG",   16 },
-  { "JPEG",    16 },
   { "YUY2",    16 },
   { "YUV422",  16 },
   { "YUV422P", 16 },
@@ -100,7 +99,6 @@ static struct {
   { "YUV411P", 12 },
   { "RGB565",  16 },
   { "RGB555",  16 },
-  { "IYUV",    12 },
   { "YUV410",  10 },
   { "YUV410P", 10 },
   { "Grey",     8 },
@@ -116,34 +114,46 @@ static VideoDevice * CreateDeviceWithDefaults(PString & adjustedDeviceName,
                                               const PString & driverName,
                                               PPluginManager * pluginMgr)
 {
-  PString adjustedDriverName = driverName;
+  if (adjustedDeviceName == "*")
+    adjustedDeviceName.MakeEmpty();
 
-  if (adjustedDeviceName.IsEmpty() || adjustedDeviceName == "*") {
-    if (driverName.IsEmpty() || driverName == "*") {
+  PString adjustedDriverName = driverName;
+  if (adjustedDriverName == "*")
+    adjustedDriverName.MakeEmpty();
+
+  if (adjustedDeviceName.IsEmpty()) {
+    if (adjustedDriverName.IsEmpty()) {
       PStringArray drivers = VideoDevice::GetDriverNames(pluginMgr);
       if (drivers.IsEmpty())
         return NULL;
 
-      // Give precedence to drivers like camera grabbers, leave out the fail safe types such as NULL
-      PINDEX driverIndex;
-      for (driverIndex = drivers.GetSize()-1; driverIndex > 0; --driverIndex) {
-        static const char * lowPriorityDrivers[] = {
-          "YUVFile", "FakeVideo", "NULLOutput"
-        };
-        PINDEX i;
-        for (i = 0; i < PARRAYSIZE(lowPriorityDrivers); i++) {
-          if (drivers[driverIndex] == lowPriorityDrivers[i])
+      // Give precedence to drivers like camera grabbers, Window
+      static const char * prioritisedDrivers[] = {
+        "Window", "SDL", "DirectShow", "VideoForWindows", "V4L", "V4L2", "1394DC", "1394AVC", "BSDCAPTURE", "FakeVideo", "NULLOutput"
+      };
+      for (PINDEX i = 0; i < PARRAYSIZE(prioritisedDrivers); i++) {
+        PINDEX driverIndex = drivers.GetValuesIndex(PString(prioritisedDrivers[i]));
+        if (driverIndex != P_MAX_INDEX) {
+          PStringArray devices = VideoDevice::GetDriversDeviceNames(drivers[driverIndex]);
+          if (!devices.IsEmpty()) {
+            adjustedDeviceName = devices[0];
+            adjustedDriverName = drivers[driverIndex];
             break;
+          }
         }
-        if (i == PARRAYSIZE(lowPriorityDrivers))
-          break;
       }
-      adjustedDriverName = drivers[driverIndex];
+
+      if (adjustedDriverName.IsEmpty())
+        adjustedDriverName = drivers[0];
     }
 
-    PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
-    if (!devices.IsEmpty())
+    if (adjustedDeviceName.IsEmpty()) {
+      PStringArray devices = VideoDevice::GetDriversDeviceNames(adjustedDriverName);
+      if (devices.IsEmpty())
+        return NULL;
+
       adjustedDeviceName = devices[0];
+    }
   }
 
   return VideoDevice::CreateDeviceByName(adjustedDeviceName, adjustedDriverName, pluginMgr);
