@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 21950 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-01-27 02:26:00 +0000 (Tue, 27 Jan 2009) $
  */
 
 #include <ptlib.h>
@@ -468,6 +468,9 @@ int PServiceProcess::InternalMain(void * arg)
     return 1;
   }
 
+  // Skip past the first argument
+  arguments.Shift(1);
+
   if (!CreateControlWindow(debugMode))
     return 1;
 
@@ -478,8 +481,14 @@ int PServiceProcess::InternalMain(void * arg)
 
   if (debugMode) {
     ::SetLastError(0);
-    PError << "Service simulation started for \"" << GetName() << "\" version " << GetVersion(true) << "\n"
-              "Close window to terminate.\n" << endl;
+    PError << "Service simulation started for \"" << GetName()
+           << "\" version " << GetVersion(true)
+           << " by " << GetManufacturer()
+           << " on " << GetOSClass() << ' ' << GetOSName()
+           << " (" << GetOSVersion() << '-' << GetOSHardware()
+           << ") with PTLib (v" << GetLibVersion()
+           << ") at " << PTime().AsString("yyyy/M/d h:mm:ss.uuu")
+           << "\nClose window to terminate.\n" << endl;
   }
 
   terminationEvent = CreateEvent(NULL, true, false, GetName());
