@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20991 $
- * $Author: dsandras $
- * $Date: 2008-09-15 18:37:53 +0000 (Mon, 15 Sep 2008) $
+ * $Revision: 21519 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-14 03:23:08 +0000 (Fri, 14 Nov 2008) $
  */
 
 #define _OSUTIL_CXX
@@ -70,7 +70,7 @@
 #include <sys/mman.h>
 #endif
 
-#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined (P_AIX) || defined(__BEOS__) || defined(P_IRIX) || defined(P_QNX)
+#if defined(P_LINUX) || defined(P_SUN4) || defined(P_SOLARIS) || defined(P_FREEBSD) || defined(P_OPENBSD) || defined(P_NETBSD) || defined(P_MACOSX) || defined(P_MACOS) || defined (P_AIX) || defined(P_BEOS) || defined(P_IRIX) || defined(P_QNX)
 #include <sys/utsname.h>
 #define  HAS_UNAME
 #elif defined(P_RTEMS)
@@ -97,7 +97,7 @@ extern "C" int select(int width,
       struct timeval *timeout);
 #endif
 
-#ifdef __BEOS__
+#ifdef P_BEOS
 #include "OS.h"
 #endif
 
@@ -106,7 +106,7 @@ extern "C" int select(int width,
 
 PString PProcess::GetOSClass()
 {
-#ifndef __BEOS__
+#ifndef P_BEOS
   return PString("Unix");
 #elif defined P_VXWORKS
   return PString("VxWorks");
@@ -399,7 +399,7 @@ extern PProcess * PProcessInstance;;
 PString PX_GetThreadName(pthread_t id)
 {
   if (PProcessInstance != NULL) {
-    PWaitAndSignal m(PProcessInstance->threadMutex);
+    PWaitAndSignal m(PProcessInstance->activeThreadMutex);
     PThread & thread = PProcessInstance->activeThreads[(unsigned)id];
     return thread.GetThreadName();
   }
@@ -503,6 +503,10 @@ void PProcess::PXOnAsyncSignal(int sig)
   switch (sig) {
     case SIGINT:
     case SIGTERM:
+      if (OnInterrupt(sig == SIGTERM))
+        break;
+      // Do next case
+
     case SIGHUP:
       raise(SIGKILL);
       break;
@@ -524,7 +528,7 @@ void PProcess::PXOnSignal(int sig)
     PMemoryHeap::GetState(state);
 #endif
     PStringStream strm;
-    threadMutex.Wait();
+    activeThreadMutex.Wait();
     PINDEX i;
     strm << "===============\n";
     strm << activeThreads.GetSize() << " active threads\n";
@@ -539,7 +543,7 @@ void PProcess::PXOnSignal(int sig)
     PMemoryHeap::GetState(state);
 #endif
     strm << "===============\n";
-    threadMutex.Signal();
+    activeThreadMutex.Signal();
     fprintf(stderr, "%s", strm.GetPointer());
 #if PMEMORY_CHECK
     PMemoryHeap::SetIgnoreAllocations(oldIgnore);
